=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2007-12-04 00:34:34 +0000
+++ bzrlib/graph.py	2007-12-07 05:31:54 +0000
@@ -361,7 +361,7 @@
         """Determine whether a revision is an ancestor of another.
 
         We answer this using heads() as heads() has the logic to perform the
-        smallest number of parent looksup to determine the ancestral
+        smallest number of parent lookups to determine the ancestral
         relationship between N revisions.
         """
         return set([candidate_descendant]) == self.heads(

=== modified file 'bzrlib/switch.py'
--- bzrlib/switch.py	2007-12-07 04:51:58 +0000
+++ bzrlib/switch.py	2007-12-07 05:31:54 +0000
@@ -47,6 +47,7 @@
     try:
         tree = control.open_workingtree()
     except errors.NotBranchError, ex:
+        # Lightweight checkout and branch is no longer there
         if force:
             return
         else:
@@ -77,34 +78,37 @@
             # have been pushed to the current bound branch then
             # synchronise the local branch with the new remote branch
             # and bind to it
-            if not force and _any_local_commits(b, bound_branch):
+            possible_transports = []
+            if not force and _any_local_commits(b, possible_transports):
                 raise errors.BzrCommandError(
                     'Cannot switch as local commits found in the checkout. '
                     'Commit these to the bound branch or use --force to '
                     'throw them away.')
             b.set_bound_location(None)
-            b.pull(to_branch, overwrite=True)
+            b.pull(to_branch, overwrite=True,
+                possible_transports=possible_transports)
             b.set_bound_location(to_branch.base)
         else:
             raise errors.BzrCommandError('Cannot switch a branch, '
                 'only a checkout.')
 
 
-def _any_local_commits(this_branch, other_branch_url):
-    """Does this branch have any commits not in the other branch?"""
+def _any_local_commits(this_branch, possible_transports):
+    """Does this branch have any commits not in the master branch?"""
     last_rev = revision.ensure_null(this_branch.last_revision())
     if last_rev != revision.NULL_REVISION:
-        a_bzrdir, relpath = BzrDir.open_containing(other_branch_url)
-        other_branch = a_bzrdir.open_branch()
+        other_branch = this_branch.get_master_branch(possible_transports)
+        this_branch.lock_read()
         other_branch.lock_read()
         try:
             other_last_rev = other_branch.last_revision()
-            remote_graph = other_branch.repository.get_revision_graph(
-                other_last_rev)
-            if last_rev not in remote_graph:
+            graph = this_branch.repository.get_graph(
+                other_branch.repository)
+            if not graph.is_ancestor(last_rev, other_last_rev):
                 return True
         finally:
             other_branch.unlock()
+            this_branch.unlock()
     return False
 
 

=== modified file 'bzrlib/tests/test_switch.py'
--- bzrlib/tests/test_switch.py	2007-12-07 04:51:58 +0000
+++ bzrlib/tests/test_switch.py	2007-12-07 05:31:54 +0000
@@ -71,7 +71,7 @@
         to_branch = branch.Branch.open('branch-2')
         # Check fails without --force
         err = self.assertRaises((errors.NotBranchError,
-            errors.BzrCommandError),
+            errors.BoundBranchConnectionFailure),
             switch.switch, checkout.bzrdir, to_branch)
         switch.switch(checkout.bzrdir, to_branch, force=True)
         self.failIfExists('checkout/file-1')

